import { ILazyComponent, IRenderContext } from "../interface";
import { TYPE_ALL_LIFE_CYCLE_EVENT } from "../types";
export declare abstract class Component<GLOBAL_TYPE = {
    [key: string]: any;
}> {
    /**
     * children components
     *
     * @type {({ [key: string]: typeof Component | ILazyComponent })}
     * @memberof Component
     */
    children: {
        [key: string]: typeof Component | ILazyComponent;
    };
    /**
     * contains component mounted dom element
     *
     * @type {HTMLElement}
     * @memberof Component
     */
    element: HTMLElement;
    /**
     * contains html - is used when render method is not present
     *
     * @type {string}
     * @memberof Component
     */
    template: string;
    /**
     * Boolean value - true if component mounted
     *
     * @memberof Component
     */
    isMounted: boolean;
    constructor();
    render?(context: IRenderContext): HTMLElement;
    /**
     * called just after the constructor - can be used to listen events
     * this is similar to constructor but `this` value is proxified
     *
     * @memberof Component
     */
    onInit(): void;
    /**
     *  set state
     *
     * @param {string} key
     * @param {*} args
     * @return {*}
     * @memberof Component
     */
    setState(key: string, ...args: any[]): void;
    deleteState(key: any): void;
    /**
     * subscribe to state changes
     *
     * @param {string} propName
     * @param {(newValue, oldValue) => void} cb
     * @return {*}
     * @memberof Component
     */
    watch(propName: string, cb: (newValue: any, oldValue: any) => void): Function;
    /**
     * unsubscribe to state changes
     *
     * @param {string} propName
     * @param {(newValue, oldValue) => void} [cb]
     * @return {*}
     * @memberof Component
     */
    unwatch(propName: string, eventListener: Function): void;
    /**
     * subscribe to events
     *
     * @param {string} event
     * @param {Function} cb
     * @return {*}
     * @memberof Component
     */
    on(event: TYPE_ALL_LIFE_CYCLE_EVENT, cb: Function): any;
    on(event: string, cb: Function): any;
    /**
     * unsubscribe to events
     *
     * @param {string} event
     * @param {Function} cb
     * @memberof Component
     */
    off(event: string, eventListener: Function): void;
    /**
     * wait for an event
     *
     * @template T
     * @param {string} eventName
     * @return {*}
     * @memberof Component
     */
    waitFor<T>(eventName: TYPE_ALL_LIFE_CYCLE_EVENT): any;
    waitFor<T>(eventName: string): any;
    /**
     * emit event to all listener at a time
     *
     * @param {string} event
     * @param {*} args
     * @return {*}
     * @memberof Component
     */
    emit(event: string, ...args: any[]): Promise<any[]>;
    /**
     * linearly calls events - in case of async: wait for one's completion and then call next
     *
     * @param {string} event
     * @param {*} args
     * @return {*}
     * @memberof Component
     */
    emitLinear(event: string, ...args: any[]): Promise<any[]>;
    /**
     * find first child element using selector
     *
     * @param {string} selector
     * @return {*}
     * @memberof Component
     */
    find(selector: string): HTMLElement;
    /**
     * find all child elements using selector
     *
     * @param {string} selector
     * @return {*}
     * @memberof Component
     */
    findAll(selector: string): NodeListOf<HTMLElement>;
    /**
     * returns outer html of component
     *
     * @readonly
     * @memberof Component
     */
    get outerHTML(): string;
    /**
     * call specified formatter and return result
     *
     * @param {string} formatterName
     * @param {*} value
     * @return {*}
     * @memberof Component
     */
    format(formatterName: string, value: any): any;
    /**
     * resolve a state by string path
     *
     * useful for nested object
     * `e.g - this.resolve('user.name.first')`
     *
     *
     * @param {*} path
     * @return {*}
     * @memberof Component
     */
    getState(path: any): any;
    /**
     * global application value
     *
     * @readonly
     * @memberof Component
     */
    get global(): GLOBAL_TYPE;
    /**
     * used for events
     *
     * @private
     * @memberof Component
     */
    private _evBus_;
    /**
     * used for property watching
     *
     * @private
     * @memberof Component
     */
    private _watchBus_;
    private _app_;
    private _ob_;
    private _directive_;
    private _formatters_;
    private _props_;
    private _reactives_;
    private _watchers_;
    private _events_;
    private _childComps_;
    private __file__;
    private _computed_;
    private _timerId_;
    static shouldCheckProp: boolean;
    getMethod(methodName: string): Function;
    private _handleExp_;
    private _render_;
    private _clearAll_;
    private _initComp_;
    private _handleAttr_;
    private _handleDir_;
}
